/**
 * chartjs-chart-graph
 * https://github.com/sgratzl/chartjs-chart-graph
 *
 * Copyright (c) 2020 Samuel Gratzl <sam@sgratzl.com>
 */

import { ILineOptions, Line, Point, IPointProps, ScatterController, UpdateMode, IControllerDatasetOptions, ScriptableAndArrayOptions, IPointPrefixedOptions, IPointPrefixedHoverOptions, ILineHoverOptions, ICoreChartOptions, ICartesianScaleTypeRegistry, Chart, ChartItem, IChartConfiguration, IChartMeta } from 'chart.js';
import { HierarchyNode } from 'd3-hierarchy';

interface IEdgeLineOptions extends ILineOptions {
    directed: boolean;
    arrowHeadSize: number;
    arrowHeadOffset: number;
}
interface IEdgeLineProps extends ILineOptions {
    points: {
        x: number;
        y: number;
    }[];
}
declare class EdgeLine extends Line {
    _orientation: 'vertical' | 'radial' | 'horizontal';
    source: Point<IPointProps & {
        angle?: number;
    }>;
    target: Point<IPointProps & {
        angle?: number;
    }>;
    options: IEdgeLineOptions;
    draw(ctx: CanvasRenderingContext2D): void;
    static readonly id = "edgeLine";
    static readonly defaults: any;
    static readonly defaultRoutes: {
        [property: string]: string;
    } | undefined;
}

interface IExtendedChartMeta extends IChartMeta<Point> {
    edges: EdgeLine[];
    _parsedEdges: {
        source: number;
        target: number;
        points: {
            x: number;
            y: number;
        }[];
    }[];
}
declare class GraphController extends ScatterController {
    _cachedMeta: IExtendedChartMeta;
    _ctx: CanvasRenderingContext2D;
    _cachedDataOpts: any;
    _type: string;
    _data: any[];
    _edges: any[];
    _sharedOptions: any;
    _edgeSharedOptions: any;
    dataElementType: any;
    dataElementOptions: any;
    private _scheduleResyncLayoutId;
    edgeElementOptions: any;
    edgeElementType: any;
    private readonly _edgeListener;
    initialize(): void;
    parse(start: number, count: number): void;
    reset(): void;
    update(mode: UpdateMode): void;
    destroy(): void;
    updateEdgeElements(edges: EdgeLine[], start: number, mode: UpdateMode): void;
    updateEdgeElement(edge: EdgeLine, index: number, properties: any, mode: UpdateMode): void;
    updateElement(point: Point, index: number, properties: any, mode: UpdateMode): void;
    resolveNodeIndex(nodes: any[], ref: string | number | any): number;
    buildOrUpdateElements(): void;
    draw(): void;
    _resyncElements(): void;
    getTreeRootIndex(): number;
    getTreeRoot(): any;
    getTreeChildren(node: {
        index: number;
    }): any[];
    _parseDefinedEdge(edge: {
        source: number;
        target: number;
    }): {
        source: number;
        target: number;
        points: never[];
    };
    _parseEdges(): any;
    addElements(): void;
    _resyncEdgeElements(): void;
    _insertElements(start: number, count: number): void;
    _removeElements(start: number, count: number): void;
    _insertEdgeElements(start: number, count: number): void;
    reLayout(): void;
    resetLayout(): void;
    stopLayout(): void;
    _scheduleResyncLayout(): void;
    resyncLayout(): void;
    static readonly id: string;
    static readonly defaults: any;
}
interface IGraphDataPoint {
    parent?: number;
}
interface IGraphEdgeDataPoint {
    source: number;
    target: number;
}
interface IGraphChartControllerDatasetOptions extends IControllerDatasetOptions, ScriptableAndArrayOptions<IPointPrefixedOptions>, ScriptableAndArrayOptions<IPointPrefixedHoverOptions>, ScriptableAndArrayOptions<IEdgeLineOptions>, ScriptableAndArrayOptions<ILineHoverOptions> {
    edges: IGraphEdgeDataPoint[];
}
declare module 'chart.js' {
    enum ChartTypeEnum {
        graph = "graph"
    }
    interface IChartTypeRegistry {
        graph: {
            chartOptions: ICoreChartOptions;
            datasetOptions: IGraphChartControllerDatasetOptions;
            defaultDataPoint: IGraphDataPoint[];
            scales: keyof ICartesianScaleTypeRegistry;
        };
    }
}
declare class GraphChart<DATA extends unknown[] = IGraphDataPoint[], LABEL = string> extends Chart<'graph', DATA, LABEL> {
    static id: string;
    constructor(item: ChartItem, config: Omit<IChartConfiguration<'graph', DATA, LABEL>, 'type'>);
}

interface IForceDirectedControllerOptions {
    simulation: {
        /**
         * auto restarts the simulation upon dataset change, one can manually restart by calling: `chart.getDatasetMeta(0).controller.reLayout();`
         *
         * @default true
         */
        autoRestart: boolean;
        initialIterations: number;
        forces: {
            /**
             * center force
             * https://github.com/d3/d3-force/#centering
             *
             * @default true
             */
            center: boolean | ICenterForce;
            /**
             * collision between nodes
             * https://github.com/d3/d3-force/#collision
             *
             * @default false
             */
            collide: boolean | ICollideForce;
            /**
             * link force
             * https://github.com/d3/d3-force/#links
             *
             * @default true
             */
            link: boolean | ILinkForce;
            /**
             * link force
             * https://github.com/d3/d3-force/#many-body
             *
             * @default true
             */
            manyBody: boolean | IManyBodyForce;
            /**
             * x positioning force
             * https://github.com/d3/d3-force/#forceX
             *
             * @default false
             */
            x: boolean | IForceXForce;
            /**
             * y positioning force
             * https://github.com/d3/d3-force/#forceY
             *
             * @default false
             */
            y: boolean | IForceYForce;
            /**
             * radial positioning force
             * https://github.com/d3/d3-force/#forceRadial
             *
             * @default false
             */
            radial: boolean | IRadialForce;
        };
    };
}
declare type ID3NodeCallback = (d: any, i: number) => number;
declare type ID3EdgeCallback = (d: any, i: number) => number;
interface ICenterForce {
    x?: number;
    y?: number;
}
interface ICollideForce {
    radius?: number | ID3NodeCallback;
    strength?: number | ID3NodeCallback;
}
interface ILinkForce {
    id?: (d: {
        source: any;
        target: any;
    }) => string | number;
    distance?: number | ID3EdgeCallback;
    strength?: number | ID3EdgeCallback;
}
interface IManyBodyForce {
    strength?: number | ID3NodeCallback;
    theta?: number;
    distanceMin?: number;
    distanceMax?: number;
}
interface IForceXForce {
    x?: number;
    strength?: number;
}
interface IForceYForce {
    y?: number;
    strength?: number;
}
interface IRadialForce {
    x?: number;
    y?: number;
    radius?: number;
    strength?: number;
}
declare class ForceDirectedGraphController extends GraphController {
    _config: IForceDirectedControllerOptions;
    private readonly _simulation;
    constructor(chart: Chart, datasetIndex: number);
    _copyPosition(): void;
    resetLayout(): void;
    resyncLayout(): void;
    reLayout(): void;
    stopLayout(): void;
    static readonly id = "forceDirectedGraph";
    static readonly defaults: any;
}
interface IForceDirectedGraphChartControllerDatasetOptions extends IGraphChartControllerDatasetOptions, IForceDirectedControllerOptions {
}
declare module 'chart.js' {
    enum ChartTypeEnum {
        forceDirectedGraph = "forceDirectedGraph"
    }
    interface IChartTypeRegistry {
        forceDirectedGraph: {
            chartOptions: ICoreChartOptions;
            datasetOptions: IForceDirectedGraphChartControllerDatasetOptions;
            defaultDataPoint: IGraphDataPoint[];
            scales: keyof ICartesianScaleTypeRegistry;
        };
    }
}
declare class ForceDirectedGraphChart<DATA extends unknown[] = IGraphDataPoint[], LABEL = string> extends Chart<'forceDirectedGraph', DATA, LABEL> {
    static id: string;
    constructor(item: ChartItem, config: Omit<IChartConfiguration<'forceDirectedGraph', DATA, LABEL>, 'type'>);
}

interface ITreeOptions {
    /**
     * tree (cluster) or dendogram layout default depends on the chart type
     */
    mode: 'dendogram' | 'tree';
    /**
     * orientation of the tree layout
     * @default horizontal
     */
    orientation: 'horizontal' | 'vertical' | 'radial';
}
declare class DendogramController extends GraphController {
    _config: {
        tree: ITreeOptions;
    };
    updateEdgeElement(line: EdgeLine, index: number, properties: any, mode: UpdateMode): void;
    updateElement(point: Point, index: number, properties: any, mode: UpdateMode): void;
    resyncLayout(): void;
    reLayout(newOptions?: Partial<ITreeOptions>): void;
    doLayout(root: HierarchyNode<{
        x: number;
        y: number;
        angle?: number;
    }>): void;
    static readonly id: string;
    static readonly defaults: any;
}
interface IDendogramChartControllerDatasetOptions extends IGraphChartControllerDatasetOptions {
    tree: ITreeOptions;
}
declare module 'chart.js' {
    enum ChartTypeEnum {
        dendogram = "dendogram"
    }
    interface IChartTypeRegistry {
        dendogram: {
            chartOptions: ICoreChartOptions;
            datasetOptions: IDendogramChartControllerDatasetOptions;
            defaultDataPoint: IGraphDataPoint[];
            scales: keyof ICartesianScaleTypeRegistry;
        };
    }
}
declare class DendogramChart<DATA extends unknown[] = IGraphDataPoint[], LABEL = string> extends Chart<'dendogram', DATA, LABEL> {
    static id: string;
    constructor(item: ChartItem, config: Omit<IChartConfiguration<'dendogram', DATA, LABEL>, 'type'>);
}
declare class TreeController extends DendogramController {
    static readonly id = "tree";
    static readonly defaults: any;
}
declare module 'chart.js' {
    enum ChartTypeEnum {
        tree = "tree"
    }
    interface IChartTypeRegistry {
        tree: {
            chartOptions: ICoreChartOptions;
            datasetOptions: IDendogramChartControllerDatasetOptions;
            defaultDataPoint: IGraphDataPoint[];
            scales: keyof ICartesianScaleTypeRegistry;
        };
    }
}
declare class TreeChart<DATA extends unknown[] = IGraphDataPoint[], LABEL = string> extends Chart<'tree', DATA, LABEL> {
    static id: string;
    constructor(item: ChartItem, config: Omit<IChartConfiguration<'tree', DATA, LABEL>, 'type'>);
}

export { DendogramChart, DendogramController, EdgeLine, ForceDirectedGraphChart, ForceDirectedGraphController, GraphChart, GraphController, ICenterForce, ICollideForce, IDendogramChartControllerDatasetOptions, IEdgeLineOptions, IEdgeLineProps, IForceDirectedControllerOptions, IForceDirectedGraphChartControllerDatasetOptions, IForceXForce, IForceYForce, IGraphChartControllerDatasetOptions, IGraphDataPoint, IGraphEdgeDataPoint, ILinkForce, IManyBodyForce, IRadialForce, ITreeOptions, TreeChart, TreeController };
//# sourceMappingURL=index.d.ts.map
